@@ -46,3 +46,8 @@ EXECUTION_MODE_HUB = "hub"
 EXECUTION_MODE_SPOKE = "spoke"
 EXECUTION_MODE_DEFAULT = EXECUTION_MODE_HUB
 EXECUTION_SPOKE_CODEBUILD_PROJECT_NAME = "servicecatalog-puppet-deploy-in-spoke"
+
+
+TASK_OUTPUTS_THAT_SHOULD_NOT_BE_DELETE = [
+    "SharePortfolioTask",
+]
